package mi.hdm.controllers;

import javafx.fxml.FXML;
import javafx.geometry.HPos;
import javafx.geometry.Pos;
import javafx.scene.Cursor;
import javafx.scene.control.*;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.*;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import mi.hdm.components.IngredientSearchResultLabel;
import mi.hdm.mealPlan.MealPlan;
import mi.hdm.recipes.*;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class MealPlanController extends BaseController {
    private final MealPlan mealPlan;
    private final RecipeManager recipeManager;
    private final RecipeSearch recipeSearch;
    private List<Recipe> searchResults;

    private final static Logger log = LogManager.getLogger(MealPlanController.class);

    @FXML
    private AnchorPane parent;
    @FXML
    private StackPane stackPane;
    @FXML
    private GridPane mealPlanGrid;
    @FXML
    private ScrollPane searchScrollPane;
    @FXML
    private VBox searchVBox;
    @FXML
    private ProgressBar caloriesProgressBar;
    @FXML
    private ProgressBar carbsProgressBar;
    @FXML
    private ProgressBar fatProgressbar;
    @FXML
    private ProgressBar proteinProgressBar;
    @FXML
    private ProgressBar fibersProgressBar;
    @FXML
    private ProgressBar saltProgressBar;
    @FXML
    private Text caloriesPercentage;
    @FXML
    private Text carbsPercentage;
    @FXML
    private Text fatPercentage;
    @FXML
    private Text proteinPercentage;
    @FXML
    private Text fiberPercentage;
    @FXML
    private Text saltPercentage;

    public MealPlanController(MealPlan mealPlan, RecipeManager recipeManager, IngredientManager ingredientManager) {
        this.mealPlan = mealPlan;
        this.recipeManager = recipeManager;
        recipeSearch = new RecipeSearch(recipeManager, ingredientManager);
        searchResults = new ArrayList<>();
    }

    @FXML
    public void initialize() {
        loadHeader(parent);
        render();
    }

    private void displayMealPlan() {
        mealPlanGrid.setGridLinesVisible(false);
        mealPlanGrid.getChildren().clear();
        mealPlanGrid.setGridLinesVisible(true);
        Font font = Font.font("System", FontWeight.BOLD, 17);
        Label labelToday = new Label("Today");
        labelToday.setFont(font);
        mealPlanGrid.add(labelToday, 0, 0);
        GridPane.setHalignment(labelToday, HPos.CENTER);
        for (int j = 1; j < 7; j++) {
            Label date = new Label(LocalDate.now().plusDays(j).format(DateTimeFormatter.ofPattern("dd.MM.")));
            date.setFont(font);
            mealPlanGrid.add(date, j, 0);
            GridPane.setHalignment(date, HPos.CENTER);
        }

        for (int j = 0; j < 7; j++) {
            final int J = j;
            mealPlan.getRecipeCodeForDay(LocalDate.now().plusDays(J)).ifPresentOrElse(code -> {
                Recipe r = recipeManager.getRecipeByCode(code);
                VBox displayRecipeVBox = new VBox();
                ImageView recipeImage = new ImageView(new Image(r.getImageURL().toString()));

                HBox recipeHBox = new HBox();
                Label name = new Label(r.getName());
                name.setFont(font);
                name.setStyle("-fx-padding: 0 50 10 10;");
                name.setCursor(Cursor.HAND);
                name.setTooltip(new Tooltip("Click to display this recipe"));
                name.setOnMouseClicked(e ->
                        changeScene(View.RECIPE_VIEW, r)
                );

                Button removeRecipeButton = new Button();
                removeRecipeButton.setStyle("-fx-background-color: dedede;" +
                        "-fx-background-radius: 5;");
                ImageView xIconImage = new ImageView(String.valueOf(Recipe.class.getResource("/images/Tasty_Pages_X_Icon.png")));
                xIconImage.setPreserveRatio(true);
                xIconImage.setFitHeight(20);
                xIconImage.setFitWidth(20);
                removeRecipeButton.setGraphic(xIconImage);
                removeRecipeButton.setOnAction(e -> {
                    mealPlan.clear(LocalDate.now().plusDays(J));
                    render();
                });

                recipeHBox.getChildren().addAll(name, removeRecipeButton);
                displayRecipeVBox.getChildren().addAll(recipeImage, recipeHBox);
                mealPlanGrid.add(displayRecipeVBox, J, 1);
                recipeImage.setPreserveRatio(true);
                recipeImage.fitWidthProperty().bind(mealPlanGrid.widthProperty().divide(7));
                recipeImage.fitHeightProperty().bind(mealPlanGrid.heightProperty().subtract(60));
            }, () -> {
                Button addRecipeButton = new Button();
                addRecipeButton.setStyle("-fx-background-color: #dedede;" +
                        "-fx-background-radius: 10;");
                ImageView imageView = new ImageView(String.valueOf(Recipe.class.getResource("/images/Tasty_Pages_Plus_Icon.png")));
                imageView.setPreserveRatio(true);
                imageView.setFitHeight(50);
                imageView.setFitWidth(50);
                addRecipeButton.setGraphic(imageView);
                mealPlanGrid.add(addRecipeButton, J, 1);
                addRecipeButton.setOnMouseClicked(e ->
                        searchRecipes(LocalDate.now().plusDays(J))
                );
                GridPane.setHalignment(addRecipeButton, HPos.CENTER);
            });
        }
    }


    public void searchRecipes(LocalDate date) {
            searchVBox = new VBox();
            searchVBox.setMaxSize(800, 800);
            searchVBox.setAlignment(Pos.CENTER);
            TextField searchTextField = new TextField();
            searchTextField.setPromptText("Search for recipe");
            searchTextField.textProperty().addListener(e -> {
                        String input = searchTextField.getText();
                        log.debug("Input in search field changed, searching through recipes with query '{}'.", input);
                        searchResults = recipeSearch.searchByQuery(recipeManager.getRecipes(), input);
                        drawRecipeSearchResults(date);
                    }
            );
            searchScrollPane = new ScrollPane();
            searchTextField.setStyle("-fx-border-color: d91c1c; -fx-border-width: 4; -fx-border-radius: 10");
            searchScrollPane.setStyle("-fx-border-color: d91c1c; -fx-border-width: 4; -fx-border-radius: 10");
            searchVBox.getChildren().addAll(searchTextField, searchScrollPane);
            stackPane.getChildren().add(searchVBox);
        }

        private void drawRecipeSearchResults(LocalDate date) {
            log.debug("Drawing search results");
            VBox resultContainer = new VBox();
            searchResults
                    .forEach(
                            result -> {
                                IngredientSearchResultLabel resultLabel = new IngredientSearchResultLabel(result);
                                resultLabel.setOnMouseClicked(e -> {
                                    log.debug("User added recipe '{}' to mealplan for day {}.", result.getName(), date);
                                    mealPlan.addRecipeToMealPlan(result, date);

                                    stackPane.getChildren().remove(searchVBox);
                                    render();
                                });
                                resultContainer.getChildren().add(resultLabel);
                            }
                    );
            searchScrollPane.setContent(resultContainer);
    }

    @FXML
    public void clearMealPlan() {
        mealPlan.clear();
        displayMealPlan();
    }

    private void drawNutritionTable() {
        log.debug("Drawing nutrition table for meal plan.");
        final Map<Nutrition, BigDecimal> weeklyNutrition = mealPlan.getNutritionTable().getTable();
        //calculate percentage of weekly recommended dosage
        double cals = weeklyNutrition.get(Nutrition.CALORIES).doubleValue() / (NutritionTable.CALORIES_DAILY * 7);
        double carbs = weeklyNutrition.get(Nutrition.CARBS).doubleValue() / (NutritionTable.CARBS_DAILY * 7);
        double fat = weeklyNutrition.get(Nutrition.FAT).doubleValue() / (NutritionTable.FAT_DAILY * 7);
        double protein = weeklyNutrition.get(Nutrition.PROTEINS).doubleValue() / (NutritionTable.DAILY_PROTEIN * 7);
        double fiber = weeklyNutrition.get(Nutrition.FIBERS).doubleValue() / (NutritionTable.DAILY_FIBER * 7);
        double salt = weeklyNutrition.get(Nutrition.SALT).doubleValue() / (NutritionTable.DAILY_SALT * 7);
        //update gui
        setPercentage(caloriesProgressBar, caloriesPercentage, cals);
        setPercentage(carbsProgressBar, carbsPercentage, carbs);
        setPercentage(fatProgressbar, fatPercentage, fat);
        setPercentage(proteinProgressBar, proteinPercentage, protein);
        setPercentage(fibersProgressBar, fiberPercentage, fiber);
        setPercentage(saltProgressBar, saltPercentage, salt);
    }

    private void setPercentage(ProgressBar progressBar, Text textObject, double value) {
        log.debug("Setting percentage for {}", textObject.getId());
        progressBar.setProgress(value);
        textObject.setText(Math.round(value * 100) + "%");
        //set text color for percentages
        if (value > 0.9) {
            if (value > 1) {
                textObject.setFill(Color.valueOf("#D91C1C")); //red
            }
            textObject.setFill(Color.valueOf("#ff8a0d")); //orange
        } else {
            textObject.setFill(Color.valueOf("#000000")); //default: black
        }
    }

    /**
     * Draws the meal plan and the nutrition table
     */
    private void render() {
        displayMealPlan();
        drawNutritionTable();
    }
}
